#include "SysMonColor.h"

SysMonColor::SysMonColor(float const x,float const y,float const z) : r(x),g(y),b(z) {
}

SysMonColor::SysMonColor(int const x,int const y,int const z) :
	r(Gamma(x/255.0)),
	g(Gamma(y/255.0)),
	b(Gamma(z/255.0)) {
}

SysMonColor::SysMonColor(rgb_color const &rgb) :
	r(Gamma(rgb.red/255.0)),
	g(Gamma(rgb.green/255.0)),
	b(Gamma(rgb.blue/255.0)) {
}

SysMonColor::SysMonColor(uint32 const rgb) :
	r(Gamma((rgb&0xff)/255.0)),
	g(Gamma(((rgb&0xff00)>>8)/255.0)),
	b(Gamma(((rgb&0xff0000)>>16)/255.0)) {
}

SysMonColor::SysMonColor(int32 const rgb) :
	r(Gamma(((rgb&0xff000000)>>24)/255.0)),
	g(Gamma(((rgb&0xff0000)>>16)/255.0)),
	b(Gamma(((rgb&0xff00)>>8)/255.0)) {
}

SysMonColor::operator uint32() const {
	int x;
	uint32 ret=0;
	x=int(255.0*InvGamma(r));
	ret|=x<<16;
	x=int(255.0*InvGamma(g));
	ret|=x<<8;
	x=int(255.0*InvGamma(b));
	ret|=x;
	return ret;
}

SysMonColor::operator rgb_color() const {
	rgb_color ret;
	ret.red=int(255.0*InvGamma(r));
	ret.green=int(255.0*InvGamma(g));
	ret.blue=int(255.0*InvGamma(b));
	return ret;
}

float SysMonColor::Gamma(float const f) {
	return (f<=0.0)?0.0:(f>=1.0)?1.0:exp(2.5*log(f));
}

float SysMonColor::InvGamma(float const f) {
	return (f<=0.0)?0.0:(f>=1.0)?1.0:exp(.4*log(f));
}

SysMonColor operator +(SysMonColor const &x,SysMonColor const &y) {
	auto SysMonColor ret;
	ret.r=x.r+y.r;
	ret.g=x.g+y.g;
	ret.b=x.b+y.b;
	return ret;
}

SysMonColor operator -(SysMonColor const &x,SysMonColor const &y) {
	auto SysMonColor ret;
	ret.r=x.r-y.r;
	ret.g=x.g-y.g;
	ret.b=x.b-y.b;
	return ret;
}

SysMonColor operator *(float const f,SysMonColor const &x) {
	auto SysMonColor ret;
	ret.r=x.r*f;
	ret.g=x.g*f;
	ret.b=x.b*f;
	return ret;
}

SysMonColor operator /(float const f,SysMonColor const &x) {
	auto SysMonColor ret;
	ret.r=x.r/f;
	ret.g=x.g/f;
	ret.b=x.b/f;
	return ret;
}

SysMonColor operator *(SysMonColor const &x,float const f) {
	auto SysMonColor ret;
	ret.r=x.r*f;
	ret.g=x.g*f;
	ret.b=x.b*f;
	return ret;
}

SysMonColor operator /(SysMonColor const &x,float const f) {
	auto SysMonColor ret;
	ret.r=x.r/f;
	ret.g=x.g/f;
	ret.b=x.b/f;
	return ret;
}
