#include "SysMonView.h"
#include "SysMonColor.h"

#include <Bitmap.h>
#include <Window.h>
#include <Message.h>

#include <stdio.h>

float SysMonMemoryUsage::Value() {
    system_info si;
    get_system_info(&si);
	return float(si.used_pages)/float(si.max_pages);
}

const int bwidth=600;
const int bheight=128;
const int bstep=1;
const int delay=100*1000;

SysMonView::SysMonView():BView(BRect(0,0,0,0),"System Monitor View",B_FOLLOW_NONE,B_PULSE_NEEDED|B_WILL_DRAW) {
	ResizeTo(bwidth-1,bheight-1);

	color[0]=SysMonColor(255,255,255);
	color[1]=SysMonColor(255,203,0);
	color[2]=SysMonColor(51,102,153);
	color[3]=SysMonColor(0,0,0);

	xbitmap=0;
	oldvalue=0.;
	bmp=new BBitmap(BRect(0,0,bwidth-1,bheight-1),B_RGB32);
	SetViewColor(B_TRANSPARENT_32_BIT);
}

void SysMonView::AttachedToWindow() {
	Window()->SetPulseRate(delay);
}

void SysMonView::Draw(BRect) {
	DrawBitmap(bmp,BRect(xbitmap,0,bwidth-1,bheight-1),BRect(0,0,bwidth-1-xbitmap,bheight-1));
	DrawBitmap(bmp,BRect(0,0,xbitmap-1,bheight-1),BRect(bwidth-xbitmap,0,bwidth-1,bheight-1));
}

void SysMonView::Pulse() {
	float newvalue=float(bheight)*SysMonMemoryUsage().Value();
	uint32* fb=(uint32*)bmp->Bits();
	long ll=bmp->BytesPerRow()/4;
	fb+=xbitmap;

	for (int x=0;x<bstep;x++) {
		float vleft=oldvalue+(newvalue-oldvalue)*x/bstep;
		float vright=oldvalue+(newvalue-oldvalue)*(x+1)/bstep;
		float topl,botl;
		if (vleft<vright) {
			topl=vright;
			botl=vleft;
		} else {
			topl=vleft;
			botl=vright;
		}
		for (int i=bheight;i>0;i--) {
			float s;
			if (i-1>=topl) {
				s=1;
			} else if (i<=botl) {
				s=0;
			} else if (i>topl) {
				if (i-1<botl) {
					s=(i-topl)+(topl-botl)/2.0;
				} else {
					s=(i-topl)+(1+(i-1-botl)/(topl-botl))/2*(topl-(i-1));
				}
			} else {
				if (i-1<botl) {
					s=((i-botl)/(topl-botl)/2)*(i-botl);
				} else {
					s=(i-.5-botl)/(topl-botl);
				}
			}
			*fb=(i/float(bheight))*(s*color[0]+(1-s)*color[2])+(1.0-i/float(bheight))*(s*color[1]+(1-s)*color[3]);
			fb+=ll;
		}
		fb+=1-ll*bheight;
	}

	oldvalue=newvalue;
	xbitmap+=bstep;
	if (xbitmap==bwidth) xbitmap=0;
	CopyBits(BRect(bstep,0,bwidth-1+bstep,bheight-1),BRect(0,0,bwidth-1,bheight-1));
}

void SysMonView::MessageReceived(BMessage*m) {
	if ((m->what<'col1')||(m->what>'col4')) {
		BView::MessageReceived(m);
		return;
	}
	color[m->what-'col1']=m->FindInt32("be:value");
}
